%
% Copyright (C) Dale L. Ang
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% Produces correctly formatted SELCON input files.
% Input is 1 CD spectrum.
%
function SSJAS()
%% Set up runtime basics
%clear;                    % Clear all variables
%dbstop if error;          % Turn on debugging
%tic;                      % Strat timer

%% Define input variables here.
%fileIs = uigetfile;
inputFilename = '784-1_01.txt';  % Self explanatory
blankFilename = 'blank.txt'; % location of the baseline buffer to subtract
convertToDe = 1; % Does the data need to be converted to deltaEpsilon
conc = 0.15; % in mg/mL
pathLength = 0.1; % in cm
numResidues = 30; % number of residues
MW = 3726; % molecular weight of protein in Da

SampleName = 'SampleName';       % Sample name - will be automated
inputPath = 'M2';    % Self explanatory 
inputFile = strcat('.',filesep,inputPath,filesep,inputFilename);
blankFile = strcat('.',filesep,inputPath,filesep,blankFilename);
inputSpec = '%f %f %f'; % This needs to be changed if there are more columns
fh = fopen(inputFile,'rt');
dataStart = 'XYDATA';
dataEnd = 'Extended';

%% Find the correct line in the input file 
lineNumStart = 0;
foundStart = false;
while ~feof(fh)
    tline = fgetl(fh);
    lineNumStart = lineNumStart + 1;
    if ischar(tline) && ~isempty(strfind(tline, dataStart))
        foundStart = true;
        break;
    end
end

lineNumEnd = lineNumStart;
foundEnd = false;
while ~feof(fh)
    tline = fgetl(fh);
    lineNumEnd = lineNumEnd + 1;
    if ischar(tline) && ~isempty(strfind(tline, dataEnd))
        foundEnd = true;
        break;
    end
end

frewind(fh);
lineNum = 0;
while ~feof(fh)
   tline = fgets(fh);
   lineNum = lineNum +1;
   if lineNum == lineNumStart
      CDCellArray = textscan(fh,inputSpec);
   end
end
fclose(fh);

%% read in the blank using the same line positions
fh = fopen(blankFile,'rt');
lineNum = 0;
while ~feof(fh)
   tline = fgets(fh);
   lineNum = lineNum +1;
   if lineNum == lineNumStart
      blankCellArray = textscan(fh,inputSpec);
   end
end
fclose(fh);

%% Process the data and subtract the blank
CDdataRaw = cell2mat(CDCellArray); % Convert to matrix
CDdataRaw2 = CDdataRaw(:,2); % Get the CD data from the experimental
blank = cell2mat(blankCellArray); % Convert to matrix
blank2 = blank(:,2); % Get the CD data from the blank
CDdataProcessed = CDdataRaw2 - blank2; % subtract the blank from the experimental

%% Do wavelength calculations
WLdata = CDdataRaw(:,1);
WLstart = WLdata(1,1);
WLend = WLdata(end);
WLstep = WLdata(1) - WLdata(2);
CDdata = CDdataProcessed;

% figure();
% hold on
% plot(CDdata);
% plot(De);

%% Convert to De
if convertToDe == 1
   MRW = MW/(numResidues-1);
   MeanResElip = CDdata*0.1*MRW/(conc*pathLength);
   De = MeanResElip/3298.2;
   CDdata = De;
end

if WLstep ~= 1
   CDdata = CDdata(1:2:end); 
end

CDdataT = CDdata'; % Transpose
CDstr = mat2str(CDdataT,4); % Convert to a string
CDstr = CDstr(2:end-1); % Strip the brackets

%% Call SELCON file generator
SSJSelProc(WLstart,WLend,WLstep,CDstr,SampleName);

fclose('all');
fprintf('Done.');
%toc;
end