%
% Copyright (C) Dale L. Ang
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% Produces correctly formatted SELCON input files.
% Input is a directory of CD spectra. Data MUST be formatted from long to short
% wavelength, ie. 240 - 190 nm and span the required training sets.
% 

function SSJASMULTI()
%% Define input variables here.
%inputFilename = '784-1_01.txt';  % for single file only
inputPath = 'M2/data';    % Self explanatory 
blankFilename = ''; % name of the baseline buffer to subtract
conc = 0.15; % in mg/mL
pathLength = 0.1; % in cm
numResidues = 30; % number of residues
MW = 3726; % molecular weight of protein in Da
convertToDe = 0; % Does the data need to be converted to deltaEpsilon? 1 = yes, 0 = no.
GenerateSELCON = 1; % Do you want to generate a SELCON input file?
SOMSpec = 0; % Do you want to process with SSNN2? 1 = yes, 0 = no.
SSWLstart = 320; % Training set start - must match map
SSWLend = 190; % Training set end - must match map

%% Don't change anything here
listing = dir(inputPath); % Lists the contents of the working directory
fileNames = {listing.name}; %
%inputFile = strcat('.',filesep,inputPath,filesep,inputFilename);
blankFile = strcat('.',filesep,inputPath,filesep,blankFilename);
inputSpec = '%f %f %f %f'; % This would need to change if there were four columns, ie. WL, CD, HT, Abs
fh = fopen(blankFile,'rt'); % Get file details from blank file
dataStart = 'XYDATA'; % Search for the start
dataEnd = 'Extended'; % Search for the end - not really necessary but handy to have

%% Find the beginning
lineNumStart = 0;
while ~feof(fh)
    tline = fgetl(fh);
    lineNumStart = lineNumStart + 1;
    if ischar(tline) && ~isempty(strfind(tline, dataStart))
        break;
    end
end

%% Find the data end
lineNumEnd = lineNumStart;
while ~feof(fh)
    tline = fgetl(fh);
    lineNumEnd = lineNumEnd + 1;
    if ischar(tline) && ~isempty(strfind(tline, dataEnd))
        break;
    end
end

%% read in the blank from lineNumStart position
frewind(fh);
lineNum = 0;
while ~feof(fh)
   tline = fgets(fh);
   lineNum = lineNum +1;
   if lineNum == lineNumStart
      blankCellArray = textscan(fh,inputSpec);
   end
end
fclose(fh);

%% Process each file
for i = 3 : (length(listing)-1)
   inputFilename = fileNames(i);
   inputFilename = cell2mat(inputFilename);
   inputFile = char(strcat('.',filesep,inputPath,filesep,inputFilename));
   
   %% CD data processing
   [CDdata,WLstart,WLend,WLstep] = SSJCDProc(inputFile,lineNumStart,inputSpec,blankCellArray);

   %% Convert to De
   if convertToDe == 1
      MRW = MW/(numResidues-1);
      MeanResElip = CDdata*0.1*MRW/(conc*pathLength);
      De = MeanResElip/3298.2;
      CDdata = De;
   end

   CDSSdata = CDdata; % make a copy to output to SOMSpec
   CDdataT = CDdata'; % Transpose
   CDstr = mat2str(CDdataT,4); % Convert to a string
   CDstr = CDstr(2:end-1); % Strip the brackets
   sampleName = inputFilename; % Samplename is the filename

   %% SELCON file generator
   if GenerateSELCON == 1
      SSJSelProc(WLstart,WLend,WLstep,CDstr,sampleName);
      figure(1);
      hold on;
      plot(CDdata);
   end
   
    %% SOMSpec preprocessing
   if SOMSpec == 1
      SSRangeStart = WLstart - SSWLstart+1;
      SSRangeEnd = WLend - SSWLend;
      SSRangeEnd = length(CDSSdata)+SSRangeEnd;
      %SSJSOMProc(CDSSNNdata,WLstart,WLend,WLstep); % do processing for SOMSpec
      SOMSpecOut(:,(i-2)) = CDSSdata(SSRangeStart:SSRangeEnd);
   end
   csvwrite('./CLAIRE/SELinput.txt',SOMSpecOut);
end

fclose('all');
fprintf('Done.');
end