%
% Copyright (C) Dale L. Ang
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% Produces correctly formatted SELCON input files.
% Input is CD spectra, prefereably of 1 nm step (although can be selected
% in the code)
%

function SSJSelProc(WLstart,WLend,WLstep,CDstr,sampleName)
%% Define input variables here.
outputFilename = 'M2_selcon';  % Self explanatory
outputPath = 'M2';    % Self explanatory 
PRINT = 0;                 % 0 or 1, 1 means more verbose output
IBasis = 7;                % 4 or 7 usually
WL_Begin = WLstart;        % Wavelength begin, eg. 250.0000
WL_End = WLend;            % Wavelength end, eg. 186.0000
Factor = 1;                % Wavelength step
IGuess = 0;                % Usually 0

%% Create matrix. Don't change anything below here.
saveFile = strcat('.',filesep,outputPath,filesep,outputFilename);
fh = fopen(saveFile,'a');

   fprintf(fh,'#                                                  \r\n');
   fprintf(fh,'# PRINT    IBasis                                  \r\n');
   fprintf(fh,'      %d         %d\r\n',PRINT,IBasis);
   fprintf(fh,'#                                                  \r\n');
   fprintf(fh,'# 1 Title Line                                     \r\n');
   fprintf(fh,' %s                                  \r\n',sampleName);
   fprintf(fh,'#                                                  \r\n');
   fprintf(fh,'#     WL_Begin     WL_End       Factor             \r\n');
   fprintf(fh,'    %3.4f    %3.4f      %1.4f\r\n',WL_Begin,WL_End,Factor);
   fprintf(fh,'#                                                  \r\n');
   fprintf(fh,'# CDDATA (Long->Short Wavelength; 260 - 178 LIMITS \r\n');
   fprintf(fh,'%s\r\n',CDstr);
   fprintf(fh,'#                                                  \r\n');
   fprintf(fh,'#  IGuess  Str1   Str2   Str3   Str4   Str5    Str6\r\n');
   fprintf(fh,'        %d                                          \r\n',IGuess);

fclose(fh);
end


