%
% Copyright (C) Dale L. Ang
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% Compares a set of spectra against the pretrained map and gives SS predictions.
% Calls SSTEStruct.m
%
%  20181017 - Final v1.2
%

function [Filenames,Parameters] = SSTEST(Filenames,Parameters)
%% Define input variables here. All other options come from the training set files.
%savePathTest = Filenames.savePathTest; %strcat('.',filesep,workingPath); % change this to desired output location
%!inputFilename = strcat(testSetFileVa,'TEST-',W); % @@@Val

if Parameters.validate == 1 % is this even used - it seems to do the same thing
   testSetFile = strcat(Filenames.savePath,Filenames.testSetFileVa,Filenames.testSetFileExt);
else
   testSetFile = strcat(Filenames.savePath,Filenames.testSetFileVa,Filenames.testSetFileExt);
end

%% Other variables - don't change unless you know what you are doing.
%structures1File = strcat(Filenames.refPathSOM,filesep,'structures1',Filenames.testSetFileExt);
coordsMatrixFile = strcat(Filenames.refPathSOM,filesep,'coordsMatrix',Filenames.testSetFileExt);

%% Read in the parameters of the SOM - direct referencing necessary in cases where there is more than one line
[Parameters] = readParameters(Filenames,Parameters);

%% Get the test file size by calling getLinesColumns function
file = testSetFile;
[numLines,numCols] = getLinesColumns(file);
Parameters.modelSize = numLines;
Parameters.testSize = numCols;

%% Load the weightMap and structures files.
weightMap = zeros(Parameters.mapLength,Parameters.mapLength,Parameters.modelSizeInt);
structures = zeros(Parameters.mapLength,Parameters.mapLength,Parameters.numStructures);

for i = 1:Parameters.modelSizeInt
   nameW = 'weightMap';
   numberW = num2str(i);
   weightMap(:,:,i) = load([Filenames.refPathSOM,filesep,nameW,numberW,Filenames.testSetFileExt]);
end

for i = 1:(Parameters.numStructures)
   nameW = 'structures';
   numberW = num2str(i);
   structures(:,:,i) = load([Filenames.refPathSOM,filesep,nameW,numberW,Filenames.testSetFileExt]);
end

% Load locations fo test set nodes
coordsMatrix = load(coordsMatrixFile);

% Create output directory
if exist(Filenames.savePathTest,'dir') ~= 7
   mkdir(Filenames.savePathTest); % Check that the output directory does not already exist.
else
   fprintf('WARNING: Output folder already exists!\nResults will be appended to existing files.\n\nCtrl-C to stop or any key to continue.\n');
   if Parameters.validate == 0
      pause;
   end
end

%% Run TESTStruct for each entry in the testSet.
% Call the structure prediction function for each 
for j = 1:Parameters.testSize
   Parameters.currentSpectrum = j;
   SSTEStruct(Filenames,Parameters,weightMap,structures,coordsMatrix);
end

%% Plot the resulting predicted secondary structures - not yet implemented in new version
% if Parameters.displayFinalTestPlots ==1
%    SSPlotting(inputFilename,savePath); 
% end

%% Finished
fprintf('Analysis complete.\n');
fclose('all'); 
%clear; % Won't work if it needs to return stuff
end