%
% Copyright (C) Dale L. Ang
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% Leave-One-Out-Validation (LOOV) program for SOMSpec
% 
% NOTE: this could all be done in memory but at the moment writes to files
%
%  20181017 - Final v1.2
%

function [Filenames,Parameters] = SSVALIDATE(Filenames,Parameters)
   tic;

   % Disable all visualisation options as they would interfere, only as a precaution.
   Parameters.realtimePlots = 0;
   Parameters.displayFinalPlots = 0;
   Parameters.displayFinalTestPlots = 0;
   Parameters.displayTestPlots = 0;
   Parameters.displayResampledSpectra = 0;
   
   refFilePrefix = Filenames.refSetFileVa;
   refPathSOM = Filenames.refPathSOM;
   refFileSavePath = strcat(Filenames.savePath);
   %@@@put in here check to make the path if it doesn't exist%
   savePathTest = Filenames.savePathTest(1:length(Filenames.savePathTest)-1);

   % Load the reference set
   Filenames.refPath = strcat(Filenames.savePath,Filenames.refSetFileVa,Filenames.refSetFileExt); % Location of input file.
   refSet = load(Filenames.refPath);

   %% Call file read function to calculate number of lines and columns
   file = Filenames.refPath; % Set the location of the file for the get LinesColumsn function
   [numLines,numCols] = getLinesColumns(file);
   Parameters.modelSize = numLines - Parameters.numStructures; % Remove the last 6 (or however many) lines from the row count
   Parameters.numCols = numCols;
   fileExt = Filenames.refSetFileExt;
   
   % Check for validity
   if Parameters.modelSize < 0
      error('Something went wrong.'); % Checks that the correct delimiter has been defined.
   end

   %% Loop over each sample in the refSet and write the TRAIN and TEST files
   for V = 1:size(refSet,2) % if the validation crashes it can be started again for a later reference by specifying here
      refSetTRAIN = refSet;

      refSetTRAIN(:,V) = [];

      % Calculate the number of rows, eg. IRset = 104, CDset = 51, SP175 = 51. Required because training set includes properties.
      refSetTEST = refSet(1:Parameters.modelSize,V); 

      % Write current refSetTRAIN and refSetTEST files
      W = num2str(V);
      Parameters.W = W;
      refSetTRAINfilename = strcat(refFilePrefix,'TRAIN','-',W,fileExt);
      Filenames.refSetFileVa = strcat(refFilePrefix,'TRAIN','-',W);
      csvwrite(strcat(refFileSavePath,refSetTRAINfilename),refSetTRAIN); % Output training dataset

      refSetTESTfilename = strcat(refFilePrefix,'TEST','-',W,fileExt);
      Filenames.testSetFileVa = strcat(refFilePrefix,'TEST','-',W);
      csvwrite(strcat(refFileSavePath,refSetTESTfilename),refSetTEST); % Output training dataset
      %csvwrite(strcat('./validate/',refSetTESTfilename),refSetTEST); % Output test dataset

      %%Fix these two
      Filenames.refPathSOM = strcat(refPathSOM,'-',W);
      Filenames.savePathTest = strcat(savePathTest,'-',W,filesep);
      %Filenames.savePathSOM = Filenames.refPathSOM; This happens in SSTRAIN
      
      [Filenames,Parameters] = SSTRAIN(Filenames,Parameters);
      if Parameters.isCancelled == 1
         disp('Cancelled by user.');
         return;
      end
      SSTEST(Filenames,Parameters);
      SSVAResults(Filenames,Parameters);
   end

   fprintf('Validation complete.\n');
   fclose('all'); 
   clear;
   toc;
end
