%
% Copyright (C) Dale L. Ang
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% Find the node with the smallest spectral NRMSD by scaling over the linspace.
% 

function [bestConc] = findOptimumScalingFactor(Parameters,weightMap,inputVectorInt)

concTestResults = zeros(length(Parameters.concError),2); % Store the concs with calculated RMSD
concTestResults(:) = 1000000; % Initialise to very large number
concTestResults(:,1) = Parameters.concError'; % Assign concErrors to first column

for c = 1:length(Parameters.concError)

   inputVectorConc = inputVectorInt.*Parameters.concError(c); % Multiply test spectrum by concError value
   weightVector = zeros(1,Parameters.modelSizeInt); % Clear node weightVector

   % Find the node with the closest spectral match by iterating over the map
   for x = 1:Parameters.mapLength
      for y = 1:Parameters.mapLength
         weightVector(:) = weightMap(x,y,:); % Read in spectrum of current node as weightVector
         d = sqrt(sum((inputVectorConc - weightVector).^2)); % Calculate RMSD of node to inputSpectrum.
         if d < concTestResults(c,2)
            concTestResults(c,2) = d; % Record RMSD against the concMultiplier
         end
      end
   end

end % end of bestConc loop

[~,RMSDindex] = min(concTestResults); % Obtain the index of the lowest RMSD value
bestConc = concTestResults(RMSDindex(2),1); % Assign the concMultiplier

end