%
% Copyright (C) Dale L. Ang
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% Generates and writes out the plots as a PDF.
% 

function generateTestPlots(Parameters,Filenames,bestX,bestY,coordsMatrix,inputVectorIntConc,BMUAver,residuals,spectNRMSD)
   maxNM = 1;
   finish = maxNM+Parameters.modelSizeInt-1;   

   % Read in coords of BMUs, to visualise on the map - handy to see if they're at the edge.
   positionBMU = zeros(Parameters.numBMUS+Parameters.trainingSetSize,2);
   for i = 1:Parameters.numBMUS
      positionBMU(i,1:2) = [bestX(i), bestY(i)];
   end

   % Get the coordinates of the rest of the nodes.
   for i = 1:Parameters.trainingSetSize
      positionBMU(Parameters.numBMUS+i,1:2) = coordsMatrix(i,2:3);
   end

   % Plot the positions of the BMUs.
   Xcoord = positionBMU(:,1)';
   Ycoord = positionBMU(:,2)';

   % Generate the plots for each test protein.
   if Parameters.displayTestPlots == 0
      figure('Visible','Off');
   else
      figure;
   end

   %% Upper plot - map with BMUs
   set(gcf,'units','centimeters','position',[1,1,18,24]) ; % x0,y0 = position, width, height
   subplot(3,1,[1 2]);
   sz = 30; % Set the dot size
   scatter(Xcoord, Ycoord,sz,'blue','filled');
   %pbaspect([2 1 1]); % x y z relative size
   title('BMU map: SOM (R), test (U)', 'fontsize', 10); % Arbitrary labels, U looks nothing like R.
   xlim([1 Parameters.mapLength]);
   ylim([1 Parameters.mapLength]);
   xlabel('X', 'fontsize', 8)
   ylabel('Y', 'fontsize', 8)

   % Location of BMUs corresponding to test spectrum
   name = '   U'; % offset for better placement
   for j = 1:Parameters.numBMUS
       numb = num2str(j);
       displayOnPlot = strcat(name,numb);
       text(positionBMU(j,1),positionBMU(j,2),displayOnPlot,'Color','red', 'HorizontalAlignment','left','fontsize',6);
   end

   % SOM node locations of training set
   name = '   R'; % offset for better placement
   nameVect = (1:Parameters.refSize);
   for j = 1:Parameters.trainingSetSize
      numb = num2str(nameVect(j));
      displayOnPlot = strcat(name,numb);
      text(positionBMU((j+Parameters.numBMUS),1),positionBMU((j+Parameters.numBMUS),2),displayOnPlot, 'HorizontalAlignment','left','fontsize', 6);
   end

   %% Lower plot - test and predicted spectrum with residuals
   subplot(3,1,3);
   hold on;
   
   spectNRM1 = 'Spectral NRMSD =';
   spectNrmsd = num2str(spectNRMSD);
   specCat = string(strcat(spectNRM1,{' '},spectNrmsd));
   
   title({'Real (black), predicted (blue), residuals (red)',specCat}, 'fontsize', 10);
   xlim([1 Parameters.modelSizeInt]);
   
   % Set x axis tick labels
   ax = gca;
   ax.XDir = 'reverse';
   ax.XTickMode = 'manual';
   ax.XTick = [maxNM finish];
   ax.XTickLabelMode = 'manual';

   if Parameters.waveUnits == "cm-1"
      ax.XTickLabel = {num2str(Parameters.wavelengthShort) num2str(Parameters.wavelengthLong)};
      %xlabelText = strcat('Wavenumbers (',Parameters.waveUnits,')');
      %cmSuper = strcat('{',char(8722),'1})');
      %xlabelText = strcat('Wavelength (cm^',cmSuper); % U+2122 is 8722 in decimal
      xlabel('Wavenumber ($cm^{-1}$)','Interpreter','latex','FontSize',8);
   else
      ax.XTickLabel = {num2str(Parameters.wavelengthLong) num2str(Parameters.wavelengthShort)};
      xlabelText = strcat('Wavelength (',Parameters.waveUnits,')');
      xlabel(xlabelText,'fontsize', 8);
   end
   
   %xlabel({'Wavelength (',Parameters.waveUnits,')'},'fontsize', 8);
   %xlabel(xlabelText,'fontsize', 8);
   ylabel('Intensity','fontsize', 8);
   plot(maxNM:finish,inputVectorIntConc,'k'); % plot real spectrum
   plot(maxNM:finish,BMUAver,'--b'); % plot predicted spectrum
   plot(maxNM:finish,residuals,':r','LineWidth',2); % plot residuals
   plot([0 Parameters.modelSizeInt],[0 0]); % add horizontal line for reference

   % Write out plots
   extensionPdf = '.pdf';
   strCurrentSpectrum = num2str(Parameters.currentSpectrum);
   currentSpectrumName = strcat('Predicted_spectrum_',strCurrentSpectrum);
   saveas(gcf,[Filenames.savePathTest,currentSpectrumName,extensionPdf],'pdf');

   fclose('all');
end